// EVM56KNATIVE.C - C stub for accessing the serial port via JAVA / WIN32
//-----------------------------------------------------------------------------
// (C) 28.2.1997 Jrgen Hasch, hasch@t-online.de
// V1.1

#define USECOMM      // include COMM API

#include <windows.h>
//#include <commdlg.h>
#include <string.h>
#include <io.h>
#include <memory.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <fcntl.h>
#include <conio.h>

#include <StubPreamble.h>
#include "evm56k.h"

// dlcld32.c
int download(char *);

static HANDLE comdev=0;
static OVERLAPPED  osWrite, osRead ;
static verbose=1; // debug

long start_address;
unsigned char *baseptr;
unsigned char *currnt;

//------------------ local functions -----------------------
//
// send single 16 Bit word
//
void txword(int w)
{
 int i;
 i = (w & 0xff00)>>8;
 txchar(i);
 i = w & 0xff;
 txchar(i);
}


// set com port parameters
//
int initport(int baudrate)
{
  BOOL fRetVal;
  DCB dcb;

dcb.DCBlength=sizeof(DCB);
  GetCommState( comdev, &dcb);

  dcb.BaudRate =  38400;
  dcb.fBinary = TRUE ;                  // binary transmission
  dcb.fParity = FALSE ;                 // no parity check
  dcb.fOutxCtsFlow = FALSE;             // monitor CTS
  dcb.fOutxDsrFlow = FALSE;             // ignore DSR
  dcb.fDtrControl = DTR_CONTROL_ENABLE ;
  dcb.fNull = FALSE;                    // don't ignore zero
  dcb.fRtsControl = RTS_CONTROL_HANDSHAKE ;
  dcb.ByteSize =  8;
  dcb.Parity =  NOPARITY;
  dcb.StopBits = ONESTOPBIT ;
  dcb.fInX = FALSE;
  dcb.fOutX = FALSE ;

  fRetVal = SetCommState( comdev, &dcb ) ;

  EscapeCommFunction( comdev, SETDTR ) ;

 return fRetVal;
}

//
// get a single character
//
int getbyte()
{
  BOOL       fReadStat ;
  COMSTAT    ComStat ;
  DWORD      dwErrorFlags;
  DWORD      dwLength;
  char ch;

  // only try to read number of bytes in queue
  ClearCommError( comdev , &dwErrorFlags, &ComStat ) ;
  dwLength =  ComStat.cbInQue ;

   if (dwLength > 0)
	{
    fReadStat = ReadFile( comdev, &ch, 1, &dwLength, &osRead ) ;
    return ch;
    }
 return -1;
}

//
// send a single character
//
int txchar(int byte)
{
	BOOL        fWriteStat ;
    DWORD       dwBytesWritten ;
    char c = byte;

   fWriteStat = WriteFile( comdev, &c, 1,
                           &dwBytesWritten, &osWrite ) ;
  if (!fWriteStat)
    return 0;
  return 1;
}

//
// receive single character
//
int rxchar()
{
int c;
int cnt;
 c=-1;
 for (cnt=0; c == -1 && cnt<1000; cnt++ )
 {
 c=getbyte();
 }
 return c;
}

//----------------------------------------------------
//
// open serial port
//
long evm56k_open(struct Hevm56k *this ,struct Hjava_lang_String *str)
{
  COMMTIMEOUTS CommTimeOuts;
  BOOL fRetVal;
  DWORD err;
  DCB dcb;

  int len = javaStringLength(str);
  char *comport=malloc(len+1);
  javaString2CString(str,comport,len+1);
  comport[len]='\0';

   osRead.hEvent = CreateEvent( NULL,    // no security
                          TRUE,    // explicit reset req
                          FALSE,   // initial event reset
                          NULL ) ; // no name

   osWrite.hEvent = CreateEvent( NULL,    // no security
                          TRUE,    // explicit reset req
                          FALSE,   // initial event reset
                          NULL ) ; // no name

  comdev = CreateFile( comport, GENERIC_READ | GENERIC_WRITE,
              0,                    // exclusive access
              NULL,                 // no security attrs
              OPEN_EXISTING,
              FILE_ATTRIBUTE_NORMAL |
              FILE_FLAG_OVERLAPPED, // overlapped I/O
              NULL );

  if ( comdev ==(HANDLE) -1 )
    return 0;

  // get any early notifications
  SetCommMask( comdev, EV_RXCHAR ) ;

  // setup device buffers
  SetupComm( comdev, 4096, 4096 ) ;

  // purge any information in the buffer
  PurgeComm( comdev, PURGE_TXABORT | PURGE_RXABORT |
                     PURGE_TXCLEAR | PURGE_RXCLEAR ) ;

  CommTimeOuts.ReadIntervalTimeout = 0xFFFFFFFF ;
  CommTimeOuts.ReadTotalTimeoutMultiplier = 0 ;
  CommTimeOuts.ReadTotalTimeoutConstant = 1000 ;
  CommTimeOuts.WriteTotalTimeoutMultiplier = 0 ;
  CommTimeOuts.WriteTotalTimeoutConstant = 1000 ;
  SetCommTimeouts( comdev , &CommTimeOuts ) ;

/*  dcb.DCBlength=sizeof(DCB);
  GetCommState( comdev, &dcb);

  dcb.BaudRate =  38400;
  dcb.fBinary = TRUE ;                  // binary transmission
  dcb.fParity = FALSE ;                 // no parity check
  dcb.fOutxCtsFlow = FALSE;             // monitor CTS
  dcb.fOutxDsrFlow = FALSE;             // ignore DSR
  dcb.fDtrControl = DTR_CONTROL_ENABLE ;
  dcb.fNull = FALSE;                    // don't ignore zero
  dcb.fRtsControl = RTS_CONTROL_HANDSHAKE ;
  dcb.ByteSize =  8;
  dcb.Parity =  NOPARITY;
  dcb.StopBits = ONESTOPBIT ;
  dcb.fInX = FALSE;
  dcb.fOutX = FALSE ;

  fRetVal = SetCommState( comdev, &dcb ) ;

  EscapeCommFunction( comdev, SETDTR ) ;    */
  initport(38400);

 free(comport);
 return fRetVal;

}

//
// close serial port
//
long evm56k_close(struct Hevm56k *this)
{
  EscapeCommFunction( comdev, CLRDTR ) ;
  CloseHandle(comdev);
  CloseHandle( osRead.hEvent ) ;
  CloseHandle( osWrite.hEvent ) ;
  comdev=0;
  return 1;
}

//
// download CLD file from PC to DSP
// requires bootloader program on DSP flash eeprom
//
long evm56k_download(struct Hevm56k *this ,struct Hjava_lang_String *str)
{
  int len = javaStringLength(str);
  char *filename=malloc(len+1);
  javaString2CString(str,filename,len+1);
  filename[len]='\0';

  // check if com port is initalized
  if (comdev != (HANDLE)0)
    download(filename);

  free(filename);
  return 0;
}

//
// set comm baudrate
//
long evm56k_setbaud(struct Hevm56k *this,long baud)
{
  return initport(baud);
}

//
// send single character
//
long evm56k_txchar(struct Hevm56k *this ,unicode c)
{
    txchar(c);
}

//
// send single 24 Bit word
//
long evm56k_txword(struct Hevm56k *this,long w)
{
 int i;
 i = (w & 0xff0000)>>16;
 txchar(i);
 i = (w & 0xff00)>>8;
 txchar(i);
 i = w & 0xff;
 txchar(i);
}

//
// receive single character
//
unicode evm56k_rxchar(struct Hevm56k *this)
{
int c;
int cnt;
 c=-1;
 for (cnt=0; c == -1 && cnt<1000; cnt++ )
 {
 c=rxchar();
 }
 return c;
}

//
// receive single 24 Bit word
//
long evm56k_rxword(struct Hevm56k *this)
{
int c,c1,c2,c3;
int cnt;

 c1=-1;
 for (cnt=0; c1 == -1 && cnt<1000; cnt++ )
 {
 c1=rxchar();
 }
 if (c1 == -1)
    return -1;

 c2=-1;
 for (cnt=0; c2 == -1 && cnt<1000; cnt++ )
 {
 c2=rxchar();
 }
 if (c2 == -1)
    return -1;

 c3=-1;
 for (cnt=0; c3 == -1 && cnt<1000; cnt++ )
 {
 c3=rxchar();
 }
 if (c3 == -1)
    return -1;

 return 65536*c3+256*c2+c1;
}

//
// receive line
//
struct Hjava_lang_String *evm56k_rxline(struct Hevm56k *this)
{
int c;
int cnt;
char str[128];

   c=0;
   cnt=0;
  while(c != -1 && c != 0x0a)
    {
    c=rxchar();
    if (c !=-1)
       str[cnt++] = c;
    }
    str[cnt-1]=0;                         // terminate string

    return makeJavaString(str,strlen(str));
}

//
// send line
//
long evm56k_txline(struct Hevm56k *this ,struct Hjava_lang_String *str)
{
    int cnt;
  BOOL fWriteStat;
  DWORD dwBytesWritten;
  int len = javaStringLength(str);
  char *buffer = malloc(len+1);
    int i;
  javaString2CString(str,buffer,len+1);
  buffer[len]='\0';

  for ( i=0; i<len; i++)
  {
   for (  cnt=0; cnt<10000; cnt++)
   {
    dwBytesWritten+=2;
   }
   txchar(buffer[i]);
 }
  txchar(0x0d);

//   fWriteStat = WriteFile( comdev, buffer, len,
//                           &dwBytesWritten, &osWrite ) ;
// printf("fWriteStat %d\n",fWriteStat);
// printf("BytesWritten %d\n",dwBytesWritten);
 free(buffer);

  if (!fWriteStat)
    return 0;

  return 1;
}

//
// receive block of characters, n.i.
//
long evm56k_rxblock(struct Hevm56k *this,HArrayOfChar *arr,long len)
{
/*    BOOL       fReadStat ;
	COMSTAT    ComStat ;
	DWORD      dwErrorFlags;
	DWORD      dwLength;
	DWORD      dwError;
	char       szError[ 10 ] ;
	NPTTYINFO  npTTYInfo ;

	// only try to read number of bytes in queue
    ClearCommError( comdev, &dwErrorFlags, &ComStat ) ;
	dwLength = min( (DWORD) nMaxLength, ComStat.cbInQue ) ;

	if (dwLength > 0)
	{
        fReadStat = ReadFile( comdev, lpszBlock,
                            dwLength, &dwLength, NULL ) ;
		if (!fReadStat)
		{
			if (GetLastError() == ERROR_IO_PENDING)
			{
				OutputDebugString("\n\rIO Pending");
				// We have to wait for read to complete.
				// This function will timeout according to the
				// CommTimeOuts.ReadTotalTimeoutConstant variable
				// Every time it times out, check for port errors
                while(!GetOverlappedResult( comdev,
					&READ_OS( npTTYInfo ), &dwLength, TRUE ))
				{
					dwError = GetLastError();
					if(dwError == ERROR_IO_INCOMPLETE)
						// normal result if not finished
						continue;
					else
					{
						// an error occurred, try to recover
						wsprintf( szError, "<CE-%u>", dwError ) ;
						WriteTTYBlock( hWnd, szError, lstrlen( szError ) ) ;
						ClearCommError( COMDEV( npTTYInfo ), &dwErrorFlags, &ComStat ) ;
						if ((dwErrorFlags > 0) && DISPLAYERRORS( npTTYInfo ))
						{
							wsprintf( szError, "<CE-%u>", dwErrorFlags ) ;
							WriteTTYBlock( hWnd, szError, lstrlen( szError ) ) ;
						}
						break;
					}

				}

			}
			else
			{
			    // some other error occurred

			    dwLength = 0 ;
				ClearCommError( COMDEV( npTTYInfo ), &dwErrorFlags, &ComStat ) ;
				if ((dwErrorFlags > 0) && DISPLAYERRORS( npTTYInfo ))
				{
					wsprintf( szError, "<CE-%u>", dwErrorFlags ) ;
					WriteTTYBlock( hWnd, szError, lstrlen( szError ) ) ;
				}
			}
		}
	}

   return ( dwLength ) ; */
    return 0;
}

//
// n.i.
//
long evm56k_txblock(struct Hevm56k *this,HArrayOfChar *arr,long len)
{
    return 0;
}














































